@@ -1,26 +1,26 @@
-use proc_macro2::TokenStream;
 use proc_macro2::Ident;
+use proc_macro2::TokenStream;
 use quote::format_ident;
 use quote::quote;
 
-use super::{BaseType, DataItem, EnumItem, TaggedItem, generate_bare_typename};
+use super::{generate_bare_typename, BaseType, DataItem, EnumItem, TaggedItem};
 use crate::util::*;
 
 // generate
 // generate parser function implementations for the set of types of A2l
 // A2ml does not use this code, since it parses its data from intermediate GenericIfData structures
 pub(crate) fn generate(typename: &str, dataitem: &DataItem) -> TokenStream {
-    let mut result = quote!{};
+    let mut result = quote! {};
 
     match &dataitem.basetype {
-        BaseType::Enum(enumitems) => {
+        BaseType::Enum { enumitems } => {
             result.extend(generate_enum_parser(typename, enumitems));
         }
-        BaseType::Struct(structitems) => {
+        BaseType::Struct { structitems } => {
             result.extend(generate_block_parser_generic(typename, structitems, false));
         }
-        BaseType::Block(structitems, is_block) => {
-            result.extend(generate_block_parser(typename, structitems, *is_block));
+        BaseType::Block { blockitems, is_block } => {
+            result.extend(generate_block_parser(typename, blockitems, *is_block));
         }
         _ => {
             panic!("only block, struct and enum are allowed as top-level types, but {} = {:#?} was encountered", typename, dataitem);
@@ -42,18 +42,18 @@ fn generate_enum_parser(typename: &str, enumitems: &[EnumItem]) -> TokenStream {
 
         let mut version_check = quote! {};
         if let Some((min_ver, max_ver)) = enitem.version_range {
-            version_check = quote!{
+            version_check = quote! {
                 parser.check_enumitem_version(context, #entag, #min_ver, #max_ver)?;
             };
         }
 
-        match_branches.push(quote!{#entag => {
+        match_branches.push(quote! {#entag => {
             #version_check
             Ok(Self::#enident)
         }});
     }
 
-    quote!{
+    quote! {
         impl #name {
             pub(crate) fn parse(parser: &mut ParserState, context: &ParseContext) -> Result<Self, ParseError> {
                 let enumname = parser.get_identifier(context)?;
@@ -72,7 +72,7 @@ fn generate_enum_parser(typename: &str, enumitems: &[EnumItem]) -> TokenStream {
 // for elements derived from an a2ml spec, blocks are structs which occur after a tag in a TaggedUnion or TaggedStruct.
 fn generate_block_parser(typename: &str, structitems: &[DataItem], is_block: bool) -> TokenStream {
     match typename {
-        "A2ml"  | "IfData" => { quote!{} }
+        "A2ml"  | "IfData" => { quote! {} }
         _ => {
             generate_block_parser_generic(typename, structitems, is_block)
         }
@@ -98,7 +98,7 @@ fn generate_block_parser_generic(typename: &str, structitems: &[DataItem], is_bl
             }
         }
     } else {
-        quote!{
+        quote! {
             let __end_offset: u32 = 0;
         }
     };
@@ -137,17 +137,17 @@ fn generate_struct_item_fragments(structitems: &[DataItem]) -> (Vec<Ident>, Vec<
     for (idx, sitem) in structitems.iter().enumerate() {
         let is_last = idx == (structitems.len() - 1);
         match &sitem.basetype {
-            BaseType::TaggedStruct(tg_items) => {
-                itemparsers.push(generate_taggeditem_parser(tg_items, false, is_last));
-                itemnames.extend(generate_tagged_item_names(tg_items));
+            BaseType::TaggedStruct { tsitems } => {
+                itemparsers.push(generate_taggeditem_parser(tsitems, false, is_last));
+                itemnames.extend(generate_tagged_item_names(tsitems));
             }
-            BaseType::TaggedUnion(tg_items) => {
-                itemparsers.push(generate_taggeditem_parser(tg_items, true, is_last));
-                itemnames.extend(generate_tagged_item_names(tg_items));
+            BaseType::TaggedUnion { tuitems } => {
+                itemparsers.push(generate_taggeditem_parser(tuitems, true, is_last));
+                itemnames.extend(generate_tagged_item_names(tuitems));
             }
-            BaseType::Sequence(seqitem) => {
+            BaseType::Sequence { seqtype } => {
                 let itemname = format_ident!("{}", sitem.varname.clone().unwrap());
-                itemparsers.push(generate_sequence_parser(&itemname, &sitem.typename, seqitem));
+                itemparsers.push(generate_sequence_parser(&itemname, &sitem.typename, seqtype));
                 location_names.push(format_ident!("__{}_location", itemname));
                 itemnames.push(itemname);
             }
@@ -155,7 +155,7 @@ fn generate_struct_item_fragments(structitems: &[DataItem]) -> (Vec<Ident>, Vec<
                 let itemname = format_ident!("{}", sitem.varname.clone().unwrap());
                 let itemname_location = format_ident!("__{}_location", itemname);
                 let itemparser = generate_item_parser_call(&sitem.typename, &sitem.basetype);
-                itemparsers.push(quote!{let (#itemname_location, #itemname) = #itemparser;});
+                itemparsers.push(quote! {let (#itemname_location, #itemname) = #itemparser;});
                 location_names.push(itemname_location);
                 itemnames.push(itemname);
             }
@@ -163,7 +163,7 @@ fn generate_struct_item_fragments(structitems: &[DataItem]) -> (Vec<Ident>, Vec<
     }
 
     if location_names.len() == 1 {
-        itemparsers.push(quote!{ let __dummy = (); });
+        itemparsers.push(quote! { let __dummy = (); });
         location_names.push(format_ident!("__dummy"));
     }
 
@@ -185,28 +185,28 @@ fn generate_item_parser_call(typename: &Option<String>, item: &BaseType) -> Toke
         BaseType::Ulong |
         BaseType::Uint64 => {
             let intparser = get_int_parser(item);
-            quote!{{
+            quote! {{
                 let line = parser.get_current_line_offset();
                 let (value, is_hex) = #intparser(context)?;
                 ((line, is_hex), value)
             }}
         }
-        BaseType::Double => { quote!{(parser.get_current_line_offset(), parser.get_double(context)?)} }
-        BaseType::Float => { quote!{(parser.get_current_line_offset(), parser.get_float(context)?)} }
-        BaseType::Ident => { quote!{(parser.get_current_line_offset(), parser.get_identifier(context)?)} }
-        BaseType::String => { quote!{(parser.get_current_line_offset(), parser.get_string(context)?)} }
-        BaseType::Array(arraytype, dim) => {
+        BaseType::Double => { quote! {(parser.get_current_line_offset(), parser.get_double(context)?)} }
+        BaseType::Float => { quote! {(parser.get_current_line_offset(), parser.get_float(context)?)} }
+        BaseType::Ident => { quote! {(parser.get_current_line_offset(), parser.get_identifier(context)?)} }
+        BaseType::String => { quote! {(parser.get_current_line_offset(), parser.get_string(context)?)} }
+        BaseType::Array {arraytype, dim} => {
             if let BaseType::Char = arraytype.basetype {
-                quote!{parser.get_string_maxlen(context, #dim)?}
+                quote! {parser.get_string_maxlen(context, #dim)?}
             } else {
                 let itemparser = generate_item_parser_call(&arraytype.typename, &arraytype.basetype);
                 let names: Vec<Ident> = (0..(*dim)).into_iter().map(
                     |x| format_ident!("__arrayitem_{}", x)
                 ).collect();
                 let parsercalls = names.iter().map(
-                    |name| quote!{let #name = #itemparser;}
+                    |name| quote! {let #name = #itemparser;}
                 );
-                quote!{{
+                quote! {{
                     #(#parsercalls)*
                     ([ #(#names.0),* ], [ #(#names.1),*])
                 }}
@@ -215,14 +215,14 @@ fn generate_item_parser_call(typename: &Option<String>, item: &BaseType) -> Toke
         BaseType::EnumRef => {
             let typename = typename.as_ref().unwrap();
             let name = format_ident!("{}", typename);
-            quote!{ (parser.get_current_line_offset(), #name::parse(parser, context)?) }
+            quote! { (parser.get_current_line_offset(), #name::parse(parser, context)?) }
         }
         BaseType::StructRef => {
             let typename = typename.as_ref().unwrap();
             let name = format_ident!("{}", typename);
-            quote!{ (parser.get_current_line_offset(), #name::parse(parser, context, 0)?) }
+            quote! { (parser.get_current_line_offset(), #name::parse(parser, context, 0)?) }
         }
-        _ => { panic!("forbidden type: {:#?}", item); }
+        _ => panic!("forbidden type: {:#?}", item)
     }
 }
 
@@ -231,14 +231,14 @@ fn generate_item_parser_call(typename: &Option<String>, item: &BaseType) -> Toke
 // simplify the handling of the integer case in generate_item_parser_call
 fn get_int_parser(item: &BaseType) -> TokenStream {
     match item {
-        BaseType::Char => { quote!{parser.get_integer_i8} }
-        BaseType::Int => { quote!{parser.get_integer_i16} }
-        BaseType::Long => { quote!{parser.get_integer_i32} }
-        BaseType::Int64 => { quote!{parser.get_integer_i64} }
-        BaseType::Uchar => { quote!{parser.get_integer_u8} }
-        BaseType::Uint => { quote!{parser.get_integer_u16} }
-        BaseType::Ulong => { quote!{parser.get_integer_u32} }
-        BaseType::Uint64 => { quote!{parser.get_integer_u64} }
+        BaseType::Char => { quote! {parser.get_integer_i8} }
+        BaseType::Int => { quote! {parser.get_integer_i16} }
+        BaseType::Long => { quote! {parser.get_integer_i32} }
+        BaseType::Int64 => { quote! {parser.get_integer_i64} }
+        BaseType::Uchar => { quote! {parser.get_integer_u8} }
+        BaseType::Uint => { quote! {parser.get_integer_u16} }
+        BaseType::Ulong => { quote! {parser.get_integer_u32} }
+        BaseType::Uint64 => { quote! {parser.get_integer_u64} }
         _ => panic!("call of get_int_parser only allowed for integer types")
     }
 }
@@ -250,7 +250,7 @@ fn get_int_parser(item: &BaseType) -> TokenStream {
 fn generate_sequence_parser(itemname: &Ident, typename: &Option<String>, seqitem: &BaseType) -> TokenStream {
     let parserfunc = generate_item_parser_call(typename, seqitem);
     let itemname_location = format_ident!("__{}_location", itemname);
-    quote!{
+    quote! {
         let mut #itemname = Vec::new();
         let mut #itemname_location = Vec::new();
         let mut done = false;
@@ -275,7 +275,7 @@ fn generate_sequence_parser(itemname: &Ident, typename: &Option<String>, seqitem
 // Generate a TokenStream representing code to parse all the tagged items of a TaggedStruct or TaggedUnion
 fn generate_taggeditem_parser(tg_items: &[TaggedItem], is_taggedunion: bool, is_last: bool) -> TokenStream {
     // result: the TokenStream that ultimately collcts all the code fragements in this function
-    let mut result = quote!{};
+    let mut result = quote! {};
 
     // item_match_arms: the match arms of the while loop that passes each set of input tokens to the appropriate item parser
     // multiplicity_check: code fragemnts that check if references marked as required are present
@@ -288,14 +288,14 @@ fn generate_taggeditem_parser(tg_items: &[TaggedItem], is_taggedunion: bool, is_
 
     // wrap the match statement inside an if or a while loop
     if is_taggedunion {
-        result.extend(quote!{
+        result.extend(quote! {
             let mut next_tag = parser.get_next_tag(context)?;
             if next_tag.is_some() {
                 #parser_core
             }
         });
     } else {
-        result.extend(quote!{
+        result.extend(quote! {
             let mut next_tag = parser.get_next_tag(context)?;
             while next_tag.is_some() {
                 #parser_core
@@ -305,7 +305,7 @@ fn generate_taggeditem_parser(tg_items: &[TaggedItem], is_taggedunion: bool, is_
     }
 
     // now that all items have been parsed, the check if all required items are present can be performed
-    result.extend(quote!{
+    result.extend(quote! {
         #multiplicity_check
     });
 
@@ -318,11 +318,11 @@ fn generate_taggeditem_parser(tg_items: &[TaggedItem], is_taggedunion: bool, is_
 // This function generates all of the match arms of the match expression
 // In order to use the result of the match arms, we also need the definition of the generated variables.
 fn generate_taggeditem_match_arms(tg_items: &[TaggedItem]) -> (TokenStream, Vec<TokenStream>, TokenStream) {
-    let mut var_definitions = quote!{};
+    let mut var_definitions = quote! {};
     // item_match_arms: the match arms of the while loop that passes each set of input tokens to the appropriate item parser
     let mut item_match_arms = Vec::new();
     // multiplicity_check: code fragemnts that check if references marked as required are present
-    let mut multiplicity_check = quote!{};
+    let mut multiplicity_check = quote! {};
 
     for item in tg_items {
         let tmp_itemname = format_ident!("__tmp_required_{}", make_varname(&item.tag));
@@ -333,12 +333,12 @@ fn generate_taggeditem_match_arms(tg_items: &[TaggedItem]) -> (TokenStream, Vec<
 
         if item.repeat {
             // repeated items are represented as Vec<TypeName>
-            var_definitions.extend(quote!{let mut #itemname: Vec<#typename> = Vec::new();});
-            store_item = quote!{
+            var_definitions.extend(quote! {let mut #itemname: Vec<#typename> = Vec::new();});
+            store_item = quote! {
                 #itemname.push(newitem);
             };
             if item.required {
-                multiplicity_check.extend(quote!{
+                multiplicity_check.extend(quote! {
                     if #itemname.len() == 0 {
                         parser.error_or_log(ParseError::InvalidMultiplicityNotPresent(context.clone(), #tag_string.to_string()))?;
                     }
@@ -349,13 +349,13 @@ fn generate_taggeditem_match_arms(tg_items: &[TaggedItem]) -> (TokenStream, Vec<
             // they are represented directly as Typename if they are required
             if item.required {
                 // required items are first stored into a temporary variable of type Option<T>
-                var_definitions.extend(quote!{let mut #tmp_itemname: Option<#typename> = None;});
-                store_item = quote!{
+                var_definitions.extend(quote! {let mut #tmp_itemname: Option<#typename> = None;});
+                store_item = quote! {
                     parser.handle_multiplicity_error(context, tag, #tmp_itemname.is_some())?;
                     #tmp_itemname = Some(newitem);
                 };
                 // during the mutliplicity check the required item can be unwrapped from the Option
-                multiplicity_check.extend(quote!{
+                multiplicity_check.extend(quote! {
                     let #itemname = if let Some(value) = #tmp_itemname {
                         value
                     } else {
@@ -364,24 +364,24 @@ fn generate_taggeditem_match_arms(tg_items: &[TaggedItem]) -> (TokenStream, Vec<
                 });
             } else {
                 // an non-repeating item that is not required
-                var_definitions.extend(quote!{let mut #itemname: Option<#typename> = None;});
-                store_item = quote!{
+                var_definitions.extend(quote! {let mut #itemname: Option<#typename> = None;});
+                store_item = quote! {
                     parser.handle_multiplicity_error(context, tag, #itemname.is_some())?;
                     #itemname = Some(newitem);
                 };
             }
         }
 
-        let mut version_check = quote!{};
+        let mut version_check = quote! {};
         if let Some((min_ver, max_ver)) = item.version_range {
-            version_check.extend(quote!{
+            version_check.extend(quote! {
                 parser.check_block_version(context, #tag_string, #min_ver, #max_ver)?;
             });
         }
 
         let is_block_item = item.is_block;
         item_match_arms.push(
-            quote!{
+            quote! {
                 #tag_string => {
                     #version_check
                     let newitem = #typename::parse(parser, &newcontext, line_offset)?;
@@ -400,7 +400,7 @@ fn generate_taggeditem_match_arms(tg_items: &[TaggedItem]) -> (TokenStream, Vec<
 // generate the match statement for parsing TaggedStructs and TaggedUnions
 fn generate_taggeditem_parser_core(tg_items: &[TaggedItem], is_taggedunion: bool, is_last: bool, item_match_arms: &[TokenStream]) -> TokenStream {
     // default action if a tag is not recognized: step back in the tokenstream and let it be handled somewhere else
-    let mut default_match_arm = quote!{
+    let mut default_match_arm = quote! {
         if is_block {
             parser.undo_get_token();
         }
@@ -409,13 +409,13 @@ fn generate_taggeditem_parser_core(tg_items: &[TaggedItem], is_taggedunion: bool
 
     if !is_taggedunion {
         // taggedstructs use a while loop; if parsing fails the loop needs to end
-        default_match_arm.extend(quote!{break;});
+        default_match_arm.extend(quote! {break;});
     }
 
     // if this taggedstruct / taggedunion is the last element in the block
     // and this block (at runtime) is actually inside /begin ...  /end, then there is no way to let the unknown tag to be handled somewhere else
     if is_last {
-        default_match_arm = quote!{
+        default_match_arm = quote! {
             if context.inside_block {
                 parser.handle_unknown_taggedstruct_tag(context, tag, is_block, &TAG_LIST)?;
             } else {
@@ -425,10 +425,13 @@ fn generate_taggeditem_parser_core(tg_items: &[TaggedItem], is_taggedunion: bool
         };
     }
 
-    let taglist: Vec<String> = tg_items.iter().map(|item| item.tag.clone() ).collect();
+    let taglist: Vec<String> = tg_items
+        .iter()
+        .map(|item| item.tag.clone())
+        .collect();
     let taglist_len = taglist.len();
     // generate the full match statement
-    quote!{
+    quote! {
         let (token, is_block, line_offset) = next_tag.unwrap();
         let tag = parser.get_token_text(token);
         let newcontext = ParseContext::from_token(tag, token, is_block);
