@@ -10,14 +10,14 @@ pub(crate) fn generate(typename: &str, dataitem: &DataItem) -> TokenStream {
     let mut result = TokenStream::new();
 
     match &dataitem.basetype {
-        BaseType::Enum(enumitems) => {
+        BaseType::Enum { enumitems } => {
             result.extend(generate_enum_writer(typename, enumitems));
         }
-        BaseType::Struct(structitems) => {
+        BaseType::Struct { structitems } => {
             result.extend(generate_struct_writer(typename, structitems));
         }
-        BaseType::Block(structitems, _) => {
-            result.extend(generate_block_writer(typename, structitems));
+        BaseType::Block { blockitems, .. } => {
+            result.extend(generate_block_writer(typename, blockitems));
         }
         _ => {
             panic!("only block, struct and enum are allowed as top-level types, but {} = {:#?} was encountered", typename, dataitem);
@@ -37,7 +37,7 @@ fn generate_enum_writer(typename: &str, enumitems: &[EnumItem]) -> TokenStream {
         let enident = format_ident!("{}", ucname_to_typename(&item.name));
         let entag = &item.name;
         // each match arm is something like:  Self::WhateverValue => "WHATEVER_VALUE",
-        match_arms.push(quote!{Self::#enident => #entag});
+        match_arms.push(quote! {Self::#enident => #entag});
     }
 
     quote! {
@@ -55,12 +55,10 @@ fn generate_enum_writer(typename: &str, enumitems: &[EnumItem]) -> TokenStream {
 
 // generate_block_writer()
 // Choose between custom handling for A2ml and IfData and generic handling for every other block.
-fn generate_block_writer(typename: &str, structitems: &[DataItem]) -> TokenStream {
+fn generate_block_writer(typename: &str, blockitems: &[DataItem]) -> TokenStream {
     match typename {
-        "A2ml" | "IfData" => { quote!{} }
-        _ => {
-            generate_block_writer_generic(typename, structitems)
-        }
+        "A2ml" | "IfData" => quote! {},
+        _ => generate_block_writer_generic(typename, blockitems)
     }
 }
 
@@ -115,8 +113,8 @@ fn generate_block_item_writers(structitems: &[DataItem]) -> Vec<TokenStream> {
         let posliteral = Literal::usize_unsuffixed(posidx);
         let location = quote! {self.__block_info.item_location.#posliteral};
         match &item.basetype {
-            BaseType::TaggedUnion(taggeditems) |
-            BaseType::TaggedStruct(taggeditems) => {
+            BaseType::TaggedUnion { tuitems: taggeditems } |
+            BaseType::TaggedStruct { tsitems: taggeditems } => {
                 let mut tgwriters = Vec::new();
                 for tgitem in taggeditems {
                     let tag = &tgitem.tag;
@@ -124,19 +122,19 @@ fn generate_block_item_writers(structitems: &[DataItem]) -> Vec<TokenStream> {
                     let tgname_out = format_ident!("{}_out", tgname);
                     let is_block = tgitem.is_block;
                     if tgitem.repeat {
-                        tgwriters.push(quote!{
+                        tgwriters.push(quote! {
                             for #tgname in &self.#tgname {
                                 let #tgname_out = #tgname.stringify(indent + 1);
                                 tgroup.push(writer::TaggedItemInfo::build(#tag, #tgname_out, #is_block, &#tgname.__block_info));
                             }
                         });
                     } else if tgitem.required {
-                        tgwriters.push(quote!{
+                        tgwriters.push(quote! {
                             let #tgname_out = self.#tgname.stringify(indent + 1);
                             tgroup.push(writer::TaggedItemInfo::build(#tag, #tgname_out, #is_block, &self.#tgname.__block_info));
                         });
                     } else {
-                        tgwriters.push(quote!{
+                        tgwriters.push(quote! {
                             if let Some(#tgname) = &self.#tgname {
                                 let #tgname_out = #tgname.stringify(indent + 1);
                                 tgroup.push(writer::TaggedItemInfo::build(#tag, #tgname_out, #is_block, &#tgname.__block_info));
@@ -144,7 +142,7 @@ fn generate_block_item_writers(structitems: &[DataItem]) -> Vec<TokenStream> {
                         });
                     }
                 }
-                write_items.push(quote!{
+                write_items.push(quote! {
                     let mut tgroup = Vec::<writer::TaggedItemInfo>::new();
                     #(#tgwriters)*
                     writer.add_group(tgroup);
@@ -152,7 +150,7 @@ fn generate_block_item_writers(structitems: &[DataItem]) -> Vec<TokenStream> {
             }
             _ => {
                 let itemident = format_ident!("{}", item.varname.as_ref().unwrap() );
-                let write_cmd = generate_block_item_write_cmd(&item.basetype, quote!{self.#itemident}, location, 0);
+                let write_cmd = generate_block_item_write_cmd(&item.basetype, quote! {self.#itemident}, location, 0);
                 write_items.push(write_cmd);
                 posidx += 1;
             }
@@ -174,31 +172,36 @@ fn generate_block_item_write_cmd(basetype: &BaseType, itemname: TokenStream, loc
         BaseType::Ulong |
         BaseType::Long |
         BaseType::Uint64 |
-        BaseType::Int64 => { quote!{ writer.add_integer(#itemname, #location.1, #location.0); } }
+        BaseType::Int64 => { quote! { writer.add_integer(#itemname, #location.1, #location.0); } }
         BaseType::Double |
-        BaseType::Float => { quote!{ writer.add_float(#itemname, #location); } }
-        BaseType::Ident => { quote!{
+        BaseType::Float => { quote! { writer.add_float(#itemname, #location); } }
+        BaseType::Ident => { quote! {
             writer.add_str(&#itemname, #location);
         } }
-        BaseType::String => { quote!{
+        BaseType::String => { quote! {
             writer.add_quoted_string(&#itemname, #location);
         } }
-        BaseType::EnumRef => { quote!{
+        BaseType::EnumRef => { quote! {
             writer.add_str(&#itemname.to_string(), #location);
         } }
-        BaseType::StructRef => { quote!{
+        BaseType::StructRef => { quote! {
             #itemname.stringify(&mut writer);
         } }
-        BaseType::Array(arraytype, dim) => {
+        BaseType::Array {arraytype, dim} => {
             if let BaseType::Char = arraytype.basetype {
                 generate_block_item_write_cmd(&BaseType::String, itemname, location, calldepth)
             } else {
                 // assign an individual idxident based on the calldepth. This enables nested arrays.
                 let idxident = format_ident!("idx{}", calldepth);
-                let arrayelemname = quote!{ #itemname[#idxident] };
-                let locationname = quote!{ #location[#idxident] };
-                let write_cmd = generate_block_item_write_cmd(&arraytype.basetype, arrayelemname, locationname, calldepth + 1);
-                quote!{
+                let arrayelemname = quote! { #itemname[#idxident] };
+                let locationname = quote! { #location[#idxident] };
+                let write_cmd = generate_block_item_write_cmd(
+                    &arraytype.basetype,
+                    arrayelemname,
+                    locationname,
+                    calldepth + 1
+                );
+                quote! {
                     for #idxident in 0..#dim {
                         // the #write_cmd inside of this loop uses #idxident to index into the array
                         #write_cmd
@@ -206,11 +209,11 @@ fn generate_block_item_write_cmd(basetype: &BaseType, itemname: TokenStream, loc
                 }
             }
         }
-        BaseType::Sequence(seqtype) => {
+        BaseType::Sequence { seqtype } => {
             let seqitemident = format_ident!("seqitem{}", calldepth);
             let seqidxident = format_ident!("seqidx{}", calldepth);
             let default_location = generate_default_location(seqtype);
-            let seqlocation = quote!{(*#location.get(#seqidxident).unwrap_or_else(|| &#default_location))};
+            let seqlocation = quote! {(*#location.get(#seqidxident).unwrap_or_else(|| &#default_location))};
 
             // in the write_cmd all the integer basetypes need an additional dereference, because .iter().enumerate() gives us references
             let write_cmd = match **seqtype {
@@ -223,10 +226,10 @@ fn generate_block_item_write_cmd(basetype: &BaseType, itemname: TokenStream, loc
                 BaseType::Long |
                 BaseType::Ulong |
                 BaseType::Int64 |
-                BaseType::Uint64 => generate_block_item_write_cmd(seqtype, quote!{*#seqitemident}, seqlocation, calldepth + 1),
-                _                => generate_block_item_write_cmd(seqtype, quote!{#seqitemident}, seqlocation, calldepth + 1),
+                BaseType::Uint64 => generate_block_item_write_cmd(seqtype, quote! {*#seqitemident}, seqlocation, calldepth + 1),
+                _                => generate_block_item_write_cmd(seqtype, quote! {#seqitemident}, seqlocation, calldepth + 1),
             };
-            quote!{
+            quote! {
                 for (#seqidxident, #seqitemident) in #itemname.iter().enumerate() {
                     #write_cmd
                 }
@@ -248,8 +251,8 @@ fn generate_default_location(basetype: &BaseType) -> TokenStream {
         BaseType::Long |
         BaseType::Ulong |
         BaseType::Int64 |
-        BaseType::Uint64 => quote!{(0, false)},
-        _ => quote!{0}
+        BaseType::Uint64 => quote! {(0, false)},
+        _ => quote! {0}
     }
 }
 
